#include <wiringSerial.h>
#include <unistd.h>
#include <stdio.h>

int main(void) {
    int fd;     
    fd = serialOpen("/dev/ttyS0", 115200);    // Turn on the serial port device and set the baud rate to 115200
    if (fd < 0) {
        fprintf(stderr, "Serial port device ttyS0 cannot be enabled\n");    // Serial port device ttyS0 cannot be enabled
        return 1;
    }
    printf("Enabling serial port device ttyS0 succeeded\n");

    char cmd[] = {0x01, 0x03, 0x00, 0x00, 0x00, 0x02, 0xC4, 0x0B};
    char buffer[10];

    while (1) {
        write(fd, cmd, 8);      // Send the command to get the signal
        usleep(7000);         // Wait

        int cnt = serialDataAvail(fd);     // Check whether there is data in the serial buffer
        if (cnt > 8) {                     // The response data length is 9 bytes
            read(fd, buffer, 9);           // Read the response data

            if (buffer[0]==0x01 && buffer[1]==0x03 && buffer[2]==0x04) {     // Check whether the returned value is the correct response data
                int distance = buffer[4] + buffer[3]*256;    // // Distance data is stored in the third and fourth bytes, with the high position in front and the low position in back
                int strength = buffer[6] + buffer[5]*256;    // The signal strength data is stored in the fifth and sixth bytes, with the high position in front and the low position in back
                printf("TFmini-i Raspberry PI C code part:\n");
                printf("Distance: %dcm\n", distance);
                printf("Strength: %d\n", strength);
            }
        }
    }

    serialClose(fd);       // Close the serial port
    return 0;
}